/**
	ADF4351-628v2.cpp
	Firmware pour piloter un circuit DDS : ADF4351
	avec un AVR atmega2560
**/

/*
	Ce fichier source "GeneUHF_ADF4351.cpp" est libre, vous pouvez le redistribuer et/ou le modifier selon
	les termes de la Licence Publique Générale GNU.

	Un grand merci à Alain Fort F1CJN feb 2,2016
	pour son travail :
	
	"ADF4251 and Arduino
	update march 7, 2016 (ROBOT V1.1 and V1.0)"
	-http://f6kbf.free.fr/html/ADF4351%20and%20Arduino_Fr_Gb.htm
	-http://f6kbf.free.fr/html/ADF4351_LCD_07032016.zip
	
	dont je me suis servi pour créer cette objet (class) en C++
	J'ai aussi utilisé mon propre travail effectué pour les DDS AD9850 et AD9951

	Silicium628 - Juillet 2018
	
*/

#include <avr/io.h>
#include <util/delay.h>
#include "ADF4351-628v2.h"

#define bitSet(value, bit) ((value) |= (1UL << (bit)))  // 1UL signifie la valeur 1 au format Unsigned Long
#define bitClear(value, bit) ((value) &= ~(1UL << (bit)))
#define bitWrite(value, bit, bitvalue) (bitvalue ? bitSet(value, bit) : bitClear(value, bit))

// Constructeur
ADF4351::ADF4351() { }


void ADF4351::init()
{
	set_LE();
	Write_All_Register();
	PFDRFout=25;
	RFint=7000;
	RFout = RFint/100 ; // fréquence de sortie
	OutputChannelSpacing = 0.01; // Pas de fréquence = 10kHz
}


void ADF4351::impulse_clk()
{
 _delay_us(5);  // temps necessaire à la stabilisation du niveau envoyé
  port_ADF4351 |=  pin_CLK;
 _delay_us(5);
  port_ADF4351 &= ~pin_CLK; // l'operateur "~" donne le complement (inverse des bits). ne pas confondre avec l'opérateur "!"
 _delay_us(5);
}


void ADF4351::impulse_LE()
{
// _delay_us(10);
  port_ADF4351 |=  pin_LE;
 _delay_us(10);

  port_ADF4351 &= ~pin_LE; 
 _delay_us(10);
}


void ADF4351::set_LE()
{
  port_ADF4351 |=  pin_LE;
}


void ADF4351::reset_LE()
{
  port_ADF4351 &= ~pin_LE; 
}


void ADF4351::set_DATA()
{
  port_ADF4351 |=  pin_DATA;
}


void ADF4351::reset_DATA()
{
  port_ADF4351 &= ~pin_DATA; 
}



void ADF4351::out1octet(uint8_t octet_i)
{
// MSB first (= mode par defaut)
	uint8_t i;
  uint8_t masque; 

	for (i=0;  i < 8; i++)  
	{
		masque = 0b10000000 >> i; // le '1' se deplace de gauche a droite
		if ( (octet_i & masque) != 0) { port_ADF4351 |=  pin_DATA; }	 else { port_ADF4351 &= ~pin_DATA; }
		impulse_clk();
	}
}



void ADF4351::Write_Register32(const uint32_t value)   //Programme un registre 32bits
{
	uint8_t octet_i;
	reset_LE();
  for (int i = 3; i >= 0; i--) // boucle sur 4 x 8bits
  {        
		octet_i=(value >> 8 * i) & 0xFF; // décalage et masquage de l'octet
		out1octet(octet_i);
	}
	impulse_LE();
}


void ADF4351::Write_All_Register()  // Programme tous les registres de l'ADF4351
{
	for (int i = 5; i >= 0; i--)  // programmation ADF4351 en commencant par R5
	{	
		 Write_Register32(registers[i]);
	}
}


void ADF4351::setFreq(double frequence) 
{
  RFout = frequence;
  
	registers[4] = 0x8C803C; // cette valeur n'est pas arbitraire. En particulier les bits 20,21,22 doivent être =0.
// ce qui est le cas : 0x8C803C = 0b 1000 1100 1000 0000 0011 1100
	if (RFout >= 2200)    		{ DIV = 1; }
	else if (RFout >= 1100)  { DIV = 2;		registers[4] += (1UL<<20); } // 1UL signifie la valeur 1 au format Unsigned Long en memoire (uint32_t -> 4octets = 32 bits)
	else if (RFout >= 550)   { DIV = 4;		registers[4] += (2UL<<20); }
	else if (RFout >= 275)   { DIV = 8;		registers[4] += (3UL<<20); }
	else if (RFout >= 137.5) { DIV = 16;		registers[4] += (4UL<<20); }
	else if (RFout >= 68.75) { DIV = 32;		registers[4] += (5UL<<20); }
											else  { DIV = 64;		registers[4] += (6UL<<20); } 

	INTA = (RFout * DIV) / PFDRFout;
	MOD = (PFDRFout / OutputChannelSpacing);
	FRACF = (((RFout * DIV) / PFDRFout) - INTA) * MOD;
	FRAC = round(FRACF); // On arrondit le résultat

	registers[0] = 0;
	registers[0] = INTA << 15; // décalage de 15 rangs pour ranger cette valeur (16 bits) à sa place dans R0 ; voir p:15 du pdf
	registers[0] = registers[0] + (FRAC << 3); // decalage de 3 rangs vers la gauche pour le rentrer dans les bits [DB3..DB14]

	registers[1] = 0;
	registers[1] = MOD << 3; // decalage de 3 pour le rentrer dans les bits [DB3..DB14]
	registers[1] = registers[1] + 1 ; // ajout de l'adresse "001"
	bitSet (registers[1], 27); // PR1-> Prescaler = 8/9 ; voir p:16 du pdf

	bitSet (registers[2], 28); // Digital lock == "110" sur b28 b27 b26
	bitSet (registers[2], 27); // digital lock 
	bitClear (registers[2], 26); // digital lock

// bitSet (registers[4], 10); //MTLD: Supply current to the RF output stage to be shut down until the part achieves lock
 
	Write_All_Register();  // Programme tous les registres de l'ADF4351
}



