//================================
// version "v10.2"
//================================

#ifndef GENE_WOBUL_ADF4351_V10_2_H
#define GENE_WOBUL_ADF4351_V10_2_H

#include <stdint.h>
#include "Arduino.h"


/** ***********************************************************************************
		CLASS AffiRect  // affiche un nombre ou un petit texte dans un rectangle						
***************************************************************************************/

class AffiRect
{
public:

	uint16_t x0; //position
	uint16_t y0;
	uint16_t dx0; //dimension
	uint16_t dy0;

//couleurs
	uint8_t Rfond, Vfond, Bfond;
	uint8_t Rcadre, Vcadre, Bcadre;
	uint8_t Rtxt, Vtxt, Btxt;
	
	AffiRect();
	
	void init(uint16_t x, uint16_t y, uint16_t dx, uint16_t dy,char txt_etiquette[10]);
	void setCouleurCadre(uint8_t *couleur_i);
	void setCouleurTxt(uint8_t *couleur_i);
	void setCouleurFond(uint8_t *couleur_i);
	void flashFond(uint8_t *couleur_i);
	void affiche_frequence_5(uint32_t F_i, uint8_t nb_dec_i, uint8_t pos_cur, uint8_t SB_i); // sur 5 digits ; SB = surbrillance 1 digit
	void affiche_frequence_6(uint32_t F_i, uint8_t nb_dec_i, uint8_t pos_cur, uint8_t SB_i); //sur 6 digits ; SB = surbrillance 1 digit
	void affiche_frequence_9(uint32_t F_i, uint8_t SB_i); //sur 9 digits ; SB = surbrillance 1 digit
	void affiche_valeur(uint32_t valeur, uint8_t nb_chiffres, char txt_unite_i[3]);
	void affiche_float(float valeur, uint8_t nb_chiffres, uint8_t nb_dec, char txt_unite_i[3]);
	void affiche_HEXA(uint32_t valeur);
	void affiche_texte(char txt_i[5]);
	
	
private:
	char txt_etiquette[13];
	char txt_unite[4];
	char texte[11];

	void traceCadre();
	void affi_etiquette();

};

/** ***********************************************************************************
		CLASS AffiV  // affiche verticalement sur fond gris						
***************************************************************************************/
class AffiV
{
public:

	uint16_t x0;
	uint16_t y0;
	uint8_t R, V, B;
	
	AffiV();

	void init(uint16_t x, uint16_t y);
	void setCouleur(uint8_t *couleur_i);
	void  affiche_texte_V(char txt_i[10]);


private:
	char texte[11];
};
	

/** *********************************************************************************** 
						CLASS LED						
***************************************************************************************/

class LED
{
public:

	uint16_t x;
	uint16_t y;

	uint8_t R;
	uint8_t G;
	uint8_t B;

	uint8_t taille; // = dimensions x et y

  LED();
  
	void init(uint16_t x_i, uint16_t y_i, uint8_t R_i, uint8_t G_i, uint8_t B_i, uint_fast8_t taille, char txt_etiquette[5]);
	void setEtat(uint8_t etat_i);  // = 0 ou 1

private:
	char txt_etiquette[5];

};


/***** autres declarations ******/


#define	PIN_ADF4351_A PINF
#define	PIN_ADF4351_B PINK


// types
struct parametres
{
	uint8_t cle;
	uint32_t frequence_AD9850;
	uint32_t frequence_ADF4351;
	uint8_t pos_curFreq_gene_AD98; // position du curseur (pour affichage freq) ->  AD9850
	uint8_t pos_curFreq_gene_ADF43; // position du curseur (pour affichage freq)	->	ADF4351
	uint8_t pos_curFreq_wobu_AD98; // position du curseur (pour affichage freq de wobulation)
	uint8_t pos_curFreq_wobu_ADF43; // position du curseur (pour affichage freq de wobulation)
	uint8_t pos_curseur_wob;  // position du curseur (pour le pas de wobulation)
	uint16_t pos_mark;
	int32_t offset_y;
	int16_t gain;
};


// fonctions

void InitADC();
void init_ports ();
void init_variables();
void init_leds_switches();
void init_afficheurs_wob();
void init_leds();
void init_leds_switches();

void save_params_to_EEPROM();
void load_params_from_EEPROM();
int freeRam();
void int_EXT_setup();
void setCouleur(uint8_t *couleur_i, uint8_t R, uint8_t V,uint8_t  B);
void inc_FRQ_AD9850();
void dec_FRQ_AD9850();
void inc_FRQ_ADF4351();
void dec_FRQ_ADF4351();
void borne_frequence_AD9850();
void borne_frequence_ADF4351();
void inc_offset();
void dec_offset();
void inc_pos_curseur_frq();
void dec_pos_curseur_frq();
void inc_pas_curseur_wob();
void dec_pas_curseur_wob();
void affiche_entete();
void affiche_freq_marqueur();
void affi_marqueur();
void affi_min_max();

void acquisition_data_WOB();
void scrute_switches();
void calcul_lambda_a();
void calcul_lambda_b();
double recadre(double valeur_i);

void affi_helps();
void trace_sinusoide();
void efface_ecran();
void trace_ecran();
void trace_pannel_graph();
void trace_afficheurs_wob();
void efface_centre_graph();

void affiche_valeurs_registres();
void affiche_lambda_a();
void affiche_lambda_b();
void affiche_params();
void affiche_freq_marqueur();
void affiche_pas_DIV(uint8_t couleur);
void affiche_V_div();
void TFT_affiche_chiffre_7seg(uint8_t num, uint16_t x_i, uint16_t y_i );
void TFT_aff_nb_form3  (uint32_t valeur, uint8_t nb_chiffres, uint8_t curseur, uint8_t R, uint8_t G, uint8_t B, uint8_t SB, uint16_t x_i, uint16_t y_i);
void TFT_aff_nb_form3b (uint32_t valeur, uint8_t nb_chiffres, uint8_t curseur, uint8_t R, uint8_t G, uint8_t B, uint8_t SB, uint16_t x_i, uint16_t y_i);


void setup();


#endif
